/*
 * Detect a Library for hardware detection
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 * Copyright (C) 2005 Alexandre Dussart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "detect.h"
#include "utils.h"
#include "serial.h"

/* mouse_detect: Detect mouses.		                              */
/************************** USB MOUSE  ******************************/
/************************** PS2 MOUSE  ******************************/
void print_info(char *mtype, char *mbus, char *device, char *model){
	if (mtype!= NULL && mbus != NULL && device != NULL){
		printf("Type:%s-%s|Port:%s\n", mbus, mtype, device); /*Dougal*/  
	}
	if(debug && model)
		printf("\t\tFound %s\n", model);
}

void mouse_detect(void){
	FILE *f;
	char *line=NULL;
	size_t len = 0;
	char *model=NULL;   /* model name of the mouse  */
	char *device=NULL;  /* device name of the mouse */
	char *mtype=NULL;  /* type of the mouse */
	char *mbus=NULL;  /* type of the mouse */
	
	if(debug)
		printf("\tProbing USB and PS/2 mouse...\n");

	if ((f = fopen("/proc/bus/input/devices", "r"))==NULL)
		return;

	if (getline(&line, &len, f) >= 0){
		for (;;) {
			if (strlen(line) < 4) {
				print_info(mtype, mbus, device, model);
				device = mbus = mtype = NULL;
				if (model) { free(model); model = NULL; }
			}
			
			if (debug) 
				printf("    line: %s", line);
			
			if (strncmp(line, "N: Name=", 8)==0){
				model = strdup(line+8);
			}
			if (strncmp(line, "P: Phys=", 8)==0){
				if (strstr(line, "usb"))
					mbus="USB";
				else
					mbus="PS2";
			}
			if (strncmp(line,"H: Handlers=", 12)==0){
				if (strncmp(line+12,"mouse", 5)==0){
					mtype="mouse";
					device = "/dev/input/mice";
				}else
					device = "Not Available";
			}
			if (getline(&line, &len, f) < 0)
				break;
		}
	}
	print_info(mtype, mbus, device, model);
	device = mbus = mtype = NULL;
	if (model) { free(model); model = NULL; }
	free(line);
	fclose(f);
}
