/*
 * Detect a Library for hardware detection
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 * Copyright (C) 2005 Alexandre Dussart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "serial.h"
#include "detect.h"
#include "utils.h"

#define NUMBER_OF_VALID_SERIAL_PORTS 4
#define SERIAL_DEVICES "/dev/ttyS"


/* Copyrigt (c) RedHat Sotfware */
/* Modified by bero and alex */
int find_legacy_modem(int fd){
  int temp, done;
  int respindex;
  int starttime;
  unsigned char resp[10], buf[2];
  struct timeval timo;
	struct termios *portattr;
	
	portattr = init_serial_port(fd);
  if (portattr) free(portattr);
  usleep(200000);

  if(write(fd, "AT\r", 3) != 3){
   /* We can write to a modem - so this is not one. */
    return 2;
  }/*endif*/
  done = 0;
  respindex = 0;
  starttime = time(NULL);
  memset(resp, 0, sizeof(resp));
  while(!done){
    timo.tv_sec = 0;
    timo.tv_usec = 250000;
    if(wait_for_input(fd, &timo) > 0){
      temp = read(fd, buf, 1);
      if(temp < 0){
        if(errno != EAGAIN){
          return 1;
        }/*endif*/
      }else{
        resp[respindex++] = buf[0];
      }/*endif*/
    }else{
      done = 1;
    }/*endif*/
    if(time(NULL) - starttime > 5){
      done = 1;
    }/*endif*/
    if(respindex > 9){
      done = 1;
    }/*endif*/
  }/*endwhile*/
  if(strstr(resp, "OK") || strchr(resp, '0')){
    return 3;
  }else{
    return 2;
  }/*endif*/
}/*endfunc find_legacy_modem */


/* Determine maximum port speed for the modem */
#define PORTSPEED(x) cfsetospeed(attr, x); cfsetispeed(attr, x); set_serial_attr(fd, attr);


typedef struct INRPAIR{
	int kernel_id;
	int speed;
}IntPair;

/*Added by bero */
long modem_speed(int fd){
	/* Port speed... */
	struct termios *attr;
	long speed = 0;
	int i;
	IntPair speeds[] = {
		{B230400, 230400},
		{B115200, 115200},
		{B57600,   57600},
		{B38400,   38400},
		{B19200,   19200},
		{B9600,     9600},
		{B4800,     4800},
		{B2400,     2400},
		{B1200,     1200},
		{0,0}};

	attr = init_serial_port(fd);
	if (attr==NULL) return 0;
	for (i=0;speeds[i].speed!=0;i++){
		PORTSPEED(speeds[i].kernel_id)
		if(find_legacy_modem(fd) == 3){
			speed = speeds[i].speed;
			break;
		}
	}
	free(attr);
	return speed;
}/*endfunc modem_speed*/

/* Copyright (c) RedHat Sotfware */
/* Dougal: make this external and run from serial_detect*/
int find_legacy_mouse(int fd){
  int modem_lines;
  int temp;
  int done;
  int starttime;
  unsigned char resp[2];
  struct timeval timo;
  struct termios portattr;

  temp = get_serial_attr(fd, &portattr);
  if(temp < 0){
    return 1;
  }/*endif*/
  temp = setup_serial_port(fd, 7, &portattr);
  if(temp < 0){
    return 1;
  }/*endif*/

  modem_lines = get_serial_lines(fd);
  modem_lines &= ~(TIOCM_RTS | TIOCM_DTR);
  set_serial_lines(fd, modem_lines);
  usleep(200000);

  modem_lines |= TIOCM_DTR | TIOCM_RTS;
  set_serial_lines(fd, modem_lines);

  done = 0;
  starttime = time(NULL);
  while(!done){
    timo.tv_sec = 0;
    timo.tv_usec = 250000;
    if(wait_for_input(fd, &timo) > 0){
      temp = read(fd, resp, 1);
      if(temp < 0){
        if(errno != EAGAIN){
          return 1;
	}/*endif*/
      }else{
        done = 1;
      }/*endif*/
    }else{
      done = 1;
    }/*endif*/
    if(time(NULL) - starttime > 2){
      done = 1;
    }/*endif*/
  }/*endwhile*/

  if(*resp == 'M'){
    return 3;
  }else{
    return 2;
  }/*endif*/
}/*endfunc find_legacy_mouse*/


int serial_is_unknown(int ttySN){
	int ret=1;
	FILE *f;
	char *line=NULL;
	size_t len = 0;

	if ((f = fopen("/proc/tty/driver/serial", "r"))){
		while (getline(&line, &len, f) >= 0) {
			if (line[1] != ':')
				continue;
			if (line[0] -'0' != ttySN)
				continue;
			if (debug)
				printf("\t\t/dev/ttyS%s", line);
			if (strncmp(line+8,"unknown",7)!=0)
				ret=0;
			break;
		}
		free(line);
		fclose(f);
	}
	return ret;
}

void serial_detect(void){
	int i;
	int fd;
	int test, testB;
	struct termios origattr;

	if(debug)
		printf("\nProbing serial ports...\n");

	/********************************************************************/
	/********************** SERIAL PORT DETECTION ***********************/
	/********************************************************************/

	/* Need to try to open /dev/ttyS<i> ports */
	for(i=0; i < NUMBER_OF_VALID_SERIAL_PORTS; i++){
		char *bktype="unknown"; /*BK*/
		long bkspeed = 0;
		char testing_port[150];
		
		snprintf(testing_port,149,"%s%d", SERIAL_DEVICES, i);

		if (serial_is_unknown(i))
			continue;

		if ((fd = open_serial_port(testing_port)) < 0)
			continue;
		
		if (get_serial_attr(fd, &origattr) < 0){
			close_serial_port(fd);
			continue;
		}

		bktype="unknown"; /*BK*/

		test = find_legacy_mouse(fd);

		if(test == 3){  /* Dougal: we found a mouse */
			printf("Type:SERIAL-mouse|Port:%s\n", testing_port);
		}else{
			testB = find_legacy_modem(fd);
			if(testB == 3)  /* we found a modem */
				bkspeed = modem_speed(fd); /*BK*/

			set_serial_attr(fd, &origattr);
			close_serial_port(fd);
			/* I don't know why the following two lines are needed...
			but if they're left out, detect doesn't find the modem
			if it's started again. */
			if((fd = open_serial_port(testing_port)) < 0){
				fprintf(stderr,
				_("\nmodem_detect(): Couldn't open previously open "
				"port %s\n"), testing_port);
				exit(1);
			}else{
				close_serial_port(fd);
			}/*endif*/

			if(debug)
				printf("\t\tFound [%s]\n", testing_port);

			if (bktype=="modem")
				printf("Type:%s|Port:%s|Speed:%ld\n", bktype, testing_port, bkspeed);
				 /*BK*/
		} /*Dougal*/
	}/*next i*/
}/*endfunc serial_detect*/


/**********************************************************************/
/* wait_for_input:  Wait for a reaction on a device attached to a     */
/*                  file descriptor.                                  */
/*                                                                    */
/*     fd: file descriptor.                                           */
/*     timeout: maximum time to wait.                                 */
/**********************************************************************/
int wait_for_input(int fd, struct timeval *timeout){
  fd_set ready;
  int n;
  
  FD_ZERO(&ready);
  FD_SET(fd, &ready);
  
  n = select(fd + 1, &ready, NULL, &ready, timeout);
  
  return n;
}/*endfunc wait_for_input*/


int open_serial_port(char *port){
  int fd;

  fd = open(port, O_RDWR | O_NONBLOCK);
  if(fd < 0){
    return fd;
  }/*endif*/
  if(fcntl(fd, F_SETFL, 0) < 0){
    close(fd);
    return -1;
  }/*endif*/
  return fd;
}/*endfunc open_serial_port*/


int close_serial_port(int fd){
  return close(fd);
}/*endfunc close_serial_port*/


int get_serial_lines(int fd){
  int modem_lines;
  
  ioctl(fd, TIOCMGET, &modem_lines);
  return modem_lines;
}/*endfunc get_serial_lines*/


int set_serial_lines(int fd, int modem_lines){
  return ioctl(fd, TIOCMSET, &modem_lines);
}/*endfunc set_serial_lines*/


int get_serial_attr(int fd, struct termios *attr){
  return tcgetattr(fd, attr);
}/*endfunc get_serial_lines*/


int set_serial_attr(int fd, struct termios *attr){
  return tcsetattr(fd, TCSANOW, attr);
}/*endfunc set_serial_attr*/


int setup_serial_port(int fd, int nbits, struct termios *attr){
  attr->c_iflag = IGNBRK | IGNPAR;
  attr->c_cflag = 0;
  attr->c_cflag &= ~(CSIZE | CSTOPB | PARENB | PARODD | PARENB);
  attr->c_cflag |= CREAD | CLOCAL;
  if(nbits == 7){
    attr->c_cflag |= CS7 | CSTOPB;
  }else{
    attr->c_cflag |= CS8;
  }/*endif*/
  attr->c_oflag = 0;
  attr->c_lflag = 0;
  
  attr->c_cc[VMIN] = 1;
  attr->c_cc[VTIME] = 5;
  
  cfsetospeed(attr, B1200);
  cfsetispeed(attr, B1200);
  return set_serial_attr(fd, attr);
}/*endfunc setup_serial_port*/


struct termios *init_serial_port(int fd){
  int modem_lines, temp;
  struct termios *portattr = (struct termios *) 
                                      my_malloc(sizeof(struct termios));
  
  /*set port to 1200 baud, 8 bits, no parity, 1 stop bit */
  temp = get_serial_attr(fd, portattr);
  if(temp < 0){
  	free(portattr);
    return (struct termios *)NULL;
  }/*endif*/
  
  /* goto 1200 baud, 8 bits */
  temp = setup_serial_port(fd, 8, portattr);
  if(temp < 0){
  	free(portattr);
    return (struct termios *)NULL;
  }/*endif*/
  
  /*set on DTR and RTS */
  modem_lines = get_serial_lines(fd);
  modem_lines |= TIOCM_RTS | TIOCM_DTR;
  set_serial_lines(fd, modem_lines);
  usleep(200000);
  return portattr;
}/*endfunc init_serial_port*/



